@@ -5,10 +5,13 @@ import os
 from configparser import ConfigParser
 from dataclasses import asdict, dataclass, field, fields
 from pathlib import Path
-from typing import Any, Iterable, no_type_check
+from typing import TYPE_CHECKING, Any, Iterable, no_type_check
 
 from pdm.compat import Distribution
 
+if TYPE_CHECKING:
+    from importlib.metadata import _SimplePath
+
 
 @dataclass
 class Setup:
@@ -252,12 +255,16 @@ class _SetupReader:
             return install_requires
 
         if isinstance(value, ast.List):
-            install_requires.extend([el.s for el in value.elts if isinstance(el, ast.Str)])
+            install_requires.extend(
+                [el.value for el in value.elts if isinstance(el, ast.Constant) and isinstance(el.value, str)]
+            )
         elif isinstance(value, ast.Name):
             variable = cls._find_variable_in_body(body, value.id)
 
             if variable is not None and isinstance(variable, ast.List):
-                install_requires.extend([el.s for el in variable.elts if isinstance(el, ast.Str)])
+                install_requires.extend(
+                    [el.value for el in variable.elts if isinstance(el, ast.Constant) and isinstance(el.value, str)]
+                )
 
         return install_requires
 
@@ -297,7 +304,9 @@ class _SetupReader:
                     val = cls._find_variable_in_body(body, val.id)
 
                 if isinstance(val, ast.List):
-                    extras_require[key.s] = [e.s for e in val.elts if isinstance(e, ast.Str)]
+                    extras_require[key.value] = [
+                        e.value for e in val.elts if isinstance(e, ast.Constant) and isinstance(e.value, str)
+                    ]
         elif isinstance(value, ast.Name):
             variable = cls._find_variable_in_body(body, value.id)
 
@@ -309,7 +318,9 @@ class _SetupReader:
                     val = cls._find_variable_in_body(body, val.id)
 
                 if isinstance(val, ast.List):
-                    extras_require[key.s] = [e.s for e in val.elts if isinstance(e, ast.Str)]
+                    extras_require[key.value] = [
+                        e.value for e in val.elts if isinstance(e, ast.Constant) and isinstance(e.value, str)
+                    ]
 
         return extras_require
 
@@ -341,13 +352,13 @@ class _SetupReader:
         if value is None:
             return None
 
-        if isinstance(value, ast.Str):
-            return value.s
+        if isinstance(value, ast.Constant) and isinstance(value.value, str):
+            return value.value
         elif isinstance(value, ast.Name):
             variable = cls._find_variable_in_body(body, value.id)
 
-            if variable is not None and isinstance(variable, ast.Str):
-                return variable.s
+            if variable is not None and isinstance(variable, ast.Constant) and isinstance(variable.value, str):
+                return variable.value
 
         return None
 
@@ -384,7 +395,7 @@ class _SetupReader:
     @staticmethod
     def _find_in_dict(dict_: ast.Dict, name: str) -> Any | None:
         for key, val in zip(dict_.keys, dict_.values):
-            if isinstance(key, ast.Str) and key.s == name:
+            if isinstance(key, ast.Constant) and key.value == name:
                 return val
         return None
 
@@ -396,24 +407,28 @@ class SetupDistribution(Distribution):
     def read_text(self, filename: str) -> str | None:
         return None
 
-    def locate_file(self, path: str | os.PathLike[str]) -> os.PathLike[str]:
+    def locate_file(self, path: str | os.PathLike[str]) -> _SimplePath:
         return Path()
 
     @property
     def metadata(self) -> dict[str, Any]:  # type: ignore[override]
         return {
-            "Name": self._data.name,
-            "Version": self._data.version,
-            "Summary": self._data.summary,
-            "Requires-Python": self._data.python_requires,
+            k: v
+            for k, v in {
+                "Name": self._data.name,
+                "Version": self._data.version,
+                "Summary": self._data.summary,
+                "Requires-Python": self._data.python_requires,
+            }.items()
+            if v is not None
         }
 
     @property
     def requires(self) -> list[str] | None:
-        from pdm.models.markers import Marker
+        from pdm.models.markers import get_marker
         from pdm.models.requirements import parse_requirement
 
-        result = self._data.install_requires
+        result = self._data.install_requires[:]
         for extra, reqs in self._data.extras_require.items():
             extra_marker = f"extra == '{extra}'"
             for req in reqs:
@@ -426,6 +441,6 @@ class SetupDistribution(Distribution):
                         new_marker = f"{old_marker} and {extra_marker}"
                 else:
                     new_marker = extra_marker
-                parsed.marker = Marker(new_marker)
+                parsed.marker = get_marker(new_marker)
                 result.append(parsed.as_line())
         return result
