@@ -1,13 +1,14 @@
 from __future__ import annotations
 
 import logging
+import os
 import re
 import subprocess
 from dataclasses import dataclass, replace
-from itertools import chain
 from pathlib import Path
 from typing import TYPE_CHECKING, Any
 
+from pdm._types import HiddenText
 from pdm.models.candidates import Candidate
 from pdm.models.markers import get_marker
 from pdm.models.repositories import Package
@@ -31,7 +32,7 @@ GIT_URL = re.compile(r"(?P<repo>[^:/]+://[^\?#]+)(?:\?rev=(?P<ref>[^#]+?))?(?:#(
 class UvResolver(Resolver):
     def __post_init__(self) -> None:
         super().__post_init__()
-        self.default_source = self.project.sources[0].url
+
         if self.locked_repository is None:
             self.locked_repository = self.project.get_locked_repository()
         if self.update_strategy not in {"reuse", "all"}:
@@ -53,22 +54,27 @@ class UvResolver(Resolver):
                 "the resolution may be inaccurate."
             )
 
-    def _build_lock_command(self) -> list[str]:
-        cmd = [*self.project.core.uv_cmd, "lock", "-p", str(self.environment.interpreter.executable)]
+    def _build_lock_command(self) -> list[str | HiddenText]:
+        cmd: list[str | HiddenText] = [
+            *self.project.core.uv_cmd,
+            "lock",
+            "-p",
+            str(self.environment.interpreter.executable),
+        ]
         if self.project.core.ui.verbosity > 0:
             cmd.append("--verbose")
         if not self.project.core.state.enable_cache:
             cmd.append("--no-cache")
         first_index = True
         for source in self.project.sources:
-            assert source.url is not None
+            url = source.url_with_credentials
             if source.type == "find_links":
-                cmd.extend(["--find-links", source.url])
+                cmd.extend(["--find-links", url])
             elif first_index:
-                cmd.extend(["--index-url", source.url])
+                cmd.extend(["--index-url", url])
                 first_index = False
             else:
-                cmd.extend(["--extra-index-url", source.url])
+                cmd.extend(["--extra-index-url", url])
         if self.project.pyproject.settings.get("resolution", {}).get("respect-source-order", False):
             cmd.append("--index-strategy=unsafe-first-match")
         else:
@@ -122,13 +128,19 @@ class UvResolver(Resolver):
                 req.specifier = get_specifier(f"=={version}")
             if marker := dep.get("marker"):
                 req.marker = get_marker(marker)
+            if extra := dep.get("extra"):
+                req.extras = extra
             return req.as_line()
 
-        def make_hash(item: dict[str, Any]) -> FileHash:
-            link = Link(item["url"])
-            if "hash" not in item:
-                item["hash"] = hash_cache.get_hash(link, session)
-            return {"url": item["url"], "file": link.filename, "hash": item["hash"]}
+        def make_hash(item: dict[str, Any], fallback_url: str | None = None) -> FileHash:
+            url = item.get("url") or fallback_url
+            if url is None:
+                raise KeyError("url")
+            link = Link(url)
+            hash_value = item.get("hash")
+            if hash_value is None:
+                hash_value = hash_cache.get_hash(link, session)
+            return {"url": url, "file": link.filename, "hash": hash_value}
 
         for package in data["package"]:
             if (
@@ -157,8 +169,11 @@ class UvResolver(Resolver):
                 req = NamedRequirement.create(name=package["name"], specifier=f"=={package['version']}")
             candidate = Candidate(req, name=package["name"], version=package["version"])
 
-            for wheel in chain(package.get("wheels", []), [sdist] if (sdist := package.get("sdist")) else []):
-                candidate.hashes.append(make_hash(wheel))
+            fallback_url = package["source"].get("url")
+            for wheel in package.get("wheels", []):
+                candidate.hashes.append(make_hash(wheel, fallback_url))
+            if sdist := package.get("sdist"):
+                candidate.hashes.append(make_hash(sdist, fallback_url))
             entry = Package(candidate, [make_requirement(dep) for dep in package.get("dependencies", [])], "")
             packages.append(entry)
             if optional_dependencies := package.get("optional-dependencies"):
@@ -185,7 +200,9 @@ class UvResolver(Resolver):
                     self.reporter.stop()
                 uv_lock_command = self._build_lock_command()
                 self.project.core.ui.echo(f"Running uv lock command: {uv_lock_command}", verbosity=Verbosity.DETAIL)
-                subprocess.run(uv_lock_command, cwd=self.project.root, check=True)
+                real_command = [s.secret if isinstance(s, HiddenText) else s for s in uv_lock_command]
+                env = {**os.environ, "UV_PROJECT_ENVIRONMENT": str(self.environment.interpreter.path.parent.parent)}
+                subprocess.run(real_command, cwd=self.project.root, check=True, env=env)
             finally:
                 if isinstance(self.reporter, RichLockReporter):
                     self.reporter.start()
